#!/usr/bin/suidperl -w
#
# E. Bollengier <ebollengier@sigma.fr> (c) 2002
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# $Id: remote_ctl.pl,v 1.9 2002/04/12 12:51:30 eric Exp $
#
# Nagios is a trademark of Ethan Galstad.
# 
#################################################################
# This cgi work with nagios 1.0
#
# INSTALL
# You must : cp remote_ctl.pl /usr/lib/cgi-bin && cd /usr/lib/cgi-bin
#            chown nagios remote_ctl.pl
#	     chmod u+s remote_ctl.pl
#	     ./remote_ctl.pl  # starting watch dog deamon
#
# USE
# This cgi allow remote server to disable service check during a backup
# for example with a :
# wget 'http://host/cgi/remote_ctl.pl?host=titi&service=web&wdog=5&action=STOP'
# and it can be restart with
# wget 'http://host/cgi/remote_ctl.pl?host=titi&service=http&action=START'
#
# During a reboot, you may use
# REBOOT(String host, Int wdog)
# BOOT(String host)
#
# To disable CHECK
# STOP(String host, String service, Int wdog)
# START(String host, String service)
# 
################################################################
# conf
################################################################

$config	= '/usr/local/nagios/etc/nagios.cfg' ;

# will be overited with config
$cmd_file = '/usr/local/nagios/var/rw/nagios.cmd' ;

$wdog_file = '/usr/local/nagios/var/rw/wdog.cmd' ;
$wdog_time = 30 ;		# read the file every 30 sec

################################################################
# don't touch please
################################################################

use CGI qw/:standard/ ;
use integer ;

my %SERVICE  ;			# service list
my %CMD ;			# host:service:action => time

################################################################
# check if remote host can do something (TODO)
################################################################
sub chk_host {
    my ($client, $host) = @_ ;
    return 1 ;
}

################################################################
# return text/plain error
################################################################
sub snd_error {
    my ($why) = @_ ;
    print "\n$why : ERREUR\n" ;
    exit 0 ;
}

################################################################
# read config files 
################################################################
sub get_config_files {
    my @config_files ;

    open (CONFIG, $config) or snd_error('CONFIG_FILE') ;
    while (defined ($ligne = <CONFIG>)) {
	chomp ($ligne) ; # Chomp the newline off the end of the line...
	
	if ($ligne =~ /^cfg_file=(.+)/) {
	    push (@config_files, $1) ;
	}
    }
    close (CONFIG) ;
    return @config_files ;
}

sub config_read {
    my ($file, $host, $serv) ;
    my @files = get_config_files() ;

    foreach $file (@files) {
	open (FILE, $file) or snd_error('CONFIG_FILE') ;
	
	while (defined ($line = <FILE>)) {
	    chomp ($line);
	    if ($line =~ /^service\[(\S+)\]=([^;]+);/) {
		$host = $1 ;
		$serv = $2 ;
                
		$SERVICE{"$1;$2"} = 1 ; 
	    }

	    if ($line =~ /^command_file=(.+)/) {
		$cmd_file = $1 ;
	    }
	}

	close (FILE) ;
    }
}

################################################################
# check if $host has $service
################################################################
sub chk_host_and_service {
    my ($host, $service) = @_ ;
    return (defined ($SERVICE{"$host;$service"})) ;
} 

################################################################
# commit cmd
################################################################
sub cmd_commit {
    my ($file, $time, $host, $service, $action) = @_ ;

    if  (-f $file) {
	snd_error('CMD_FILE') ;
    }

    open (CMD, ">$file") or snd_error('CMD_FILE') ;

    $time = $time*60 + time() ;	# time in min

    # we can add a DISABLE_ALL_NOTIFICATIONS_BEYOND_HOST

    if ($action eq 'STOP') {	# send command, and launch wdog
	chk_host_and_service($host, $service) or snd_error('HOST/SRV') ;
	print CMD "[$time] DISABLE_SVC_CHECK;$host;$service\n" ;

    } elsif ($action eq 'START') {
	chk_host_and_service($host, $service) or snd_error('HOST/SRV') ;
	print CMD "[$time] ENABLE_SVC_CHECK;$host;$service\n" ;

    } elsif ($action eq 'REBOOT') {
	print CMD "[$time] DISABLE_HOST_SVC_CHECKS;$host\n" ;
	print CMD "[$time] DISABLE_HOST_CHECK;$host\n" ;

    } elsif ($action eq 'BOOT') {
	print CMD "[$time] ENABLE_HOST_SVC_CHECKS;$host\n" ;
	print CMD "[$time] ENABLE_HOST_CHECK;$host\n" ;

    } else {
	snd_error('ACTION') ;
    }
    
    close (CMD) ;
}


# send a cmd in raw mode
sub cmd_write {
    my ($file, $cmd) = @_ ;

    if  (-f $file) {
	snd_error('CMD_FILE') ;
    }

    open (CMD, ">$file") or snd_error('CMD_FILE') ;
    print CMD $cmd ;
    close (CMD) ;
}


################################################################
# cgi mode
################################################################

sub cgi_mode {
    print header(-type=>'text/plain') ;

    $host	= param('host') ;	# host name
    $service	= param('service') ;	# service name
    $action	= param('action') ;	# START | STOP
    $wdog	= param('wdog') || '' ;	# watch dog in min
    
    $client	= remote_host() ;	# get client ip
    
    snd_error('ARGUMENT') if ($host =~ /;/) ; 
    
    # check if $client can START|STOP $host
    chk_host($client, $host) or snd_error('ACCESS') ;

    config_read() ;			# update %SERVICE
    
    if ($wdog =~ /\d+/) {
	if ($action eq 'STOP') {
	    cmd_commit($cmd_file, 0, $host, $service, 'STOP') ;
	    cmd_commit($wdog_file, $wdog, $host, $service, 'START') ;
	} elsif ($action eq 'REBOOT') {
	    cmd_commit($cmd_file, 0, $host, $service, 'REBOOT') ;
	    cmd_commit($wdog_file, $wdog, $host, $service, 'BOOT') ;
	}
    } else {
	if (($action eq 'START') or ($action eq 'BOOT')) {
	    cmd_commit($wdog_file, 0, $host, $service, $action) ;
	}
    }
    print "OK\n" ;
}


################################################################
# handle for signal (deamon watch dog)
################################################################

sub sigterm_handle {
    close (WDOG_FP) ;
    unlink($wdog_file) ;
    exit (0) ;
}

sub sigusr1_handle {
    # read the next entry (break the sleep)
}

sub sigusr2_handle { # don't work
    open (FP, ">/tmp/remote_ctl$$") or die 'Error' ;
    print FP "dump\n" ;
    foreach $val (keys %CMD) {
	print FP "[$CMD{$val}] $val\n" ;
    }
    print FP "ok\n" ;
    close (FP) ;
}

sub sighup_handle {		# reread config file
    foreach $key (keys %SERVICE) {
	delete $SERVICE{$key} ;
    }
    config_read() ;
}

################################################################
# deamon watch dog
################################################################

sub wdog_mode {

    if (fork()) {		# deamon mode
	print ("Watch dog mode, go to background\n") ;
	exit (0) ;
    }
    
    close (STDIN) ;
    close (STDOUT) ;
    close (STDERR) ;

    open (STDIN, '/dev/null') ;
    open (STDERR, '/dev/null') ;
    open (STDOUT, '/dev/null') ;

    $ENV{'PATH'} = '/bin:/usr/bin' ;

    $SIG{'TERM'} = \&sigterm_handle ;
    $SIG{'HUP'} = \&sighup_handle ;
    $SIG{'USR1'} = \&sigusr1_handle ;
    $SIG{'USR2'} = \&sigusr2_handle ;

    system("mkfifo -m 600 $wdog_file &> /dev/null") ;

    config_read() ;

    open (WDOG_FP, $wdog_file) or die "Error $wdog_file" ; 

    while (1) {
	$SIG{'USR2'} = \&sigusr2_handle ;
	sleep ($wdog_time) ;
	$SIG{'USR2'} = 'IGNORE' ;

	# read next cmd
	while ($line = <WDOG_FP>) {
	    chomp ($line) ;
	    if ($line =~ /^\[(\d+)\]\s(.+)/) {
		$CMD{$2} = $1 ;
	    }
	}
	# process cmd
	$t = time() ;

	foreach $val (keys %CMD) {
	    if ($CMD{$val} && $t >= $CMD{$val}) {
		$cmd = "[$CMD{$val}] $val\n" ;
		delete $CMD{$val} ;
		cmd_write($cmd_file, $cmd) ;
	    }
	}
    }
}

################################################################
# main
################################################################

if (param()) {
    cgi_mode() ;
} else {
    wdog_mode() ;
}

# EOF


